# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Intrepid2 clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Intrepid2 requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.17.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Intrepid2 build
## ---------------------------------------------------------------------------

set(Intrepid2_CXX_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpicxx")

set(Intrepid2_C_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpicc")

set(Intrepid2_Fortran_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpif90")
# Deprecated!
set(Intrepid2_FORTRAN_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Intrepid2 build
## ---------------------------------------------------------------------------

## Give the build type
set(Intrepid2_CMAKE_BUILD_TYPE "RelWithDebInfo")

## Set compiler flags, including those determined by build type
set(Intrepid2_CXX_FLAGS [[ ]])

set(Intrepid2_C_FLAGS [[  -O2 -Wall -U_FORTIFY_SOURCE -D_FORTIFY_SOURCE=3 -fstack-protector-strong -funwind-tables -fasynchronous-unwind-tables -fstack-clash-protection -Werror=return-type -flto=auto -g -fopenmp ]])

set(Intrepid2_Fortran_FLAGS [[ ]])
# Deprecated
set(Intrepid2_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(Intrepid2_EXTRA_LD_FLAGS [[-lgfortran]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Intrepid2_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib64/mpi/gcc/openmpi4/lib64")
set(Intrepid2_BUILD_SHARED_LIBS "ON")

set(Intrepid2_LINKER /usr/bin/ld)
set(Intrepid2_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Intrepid2_INSTALL_DIR "/usr/lib64/mpi/gcc/openmpi4")

## List of package libraries
set(Intrepid2_LIBRARIES Intrepid2::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Intrepid2_MPI_LIBRARIES "")
set(Intrepid2_MPI_LIBRARY_DIRS "")
set(Intrepid2_MPI_INCLUDE_DIRS "")
set(Intrepid2_MPI_EXEC "/usr/lib64/mpi/gcc/openmpi4/bin/mpiexec")
set(Intrepid2_MPI_EXEC_MAX_NUMPROCS "4")
set(Intrepid2_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Intrepid2_ENABLE_TeuchosCore ON)
set(Intrepid2_ENABLE_TeuchosNumerics ON)
set(Intrepid2_ENABLE_Shards ON)
set(Intrepid2_ENABLE_KokkosCore ON)
set(Intrepid2_ENABLE_KokkosContainers ON)
set(Intrepid2_ENABLE_KokkosAlgorithms ON)
set(Intrepid2_ENABLE_Sacado ON)
set(Intrepid2_ENABLE_KokkosKernels ON)
set(Intrepid2_ENABLE_TrilinosTribitsLastLib OFF)

# Exported cache variables
set(Intrepid2_ENABLE_DEBUG "OFF")
set(HAVE_INTREPID2_DEBUG "OFF")
set(Intrepid2_ENABLE_DEBUG_INF_CHECK "OFF")
set(HAVE_INTREPID2_DEBUG_INF_CHECK "OFF")

# Include configuration of dependent packages
if (NOT TARGET TeuchosCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosCore/TeuchosCoreConfig.cmake")
endif()
if (NOT TARGET TeuchosNumerics::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosNumerics/TeuchosNumericsConfig.cmake")
endif()
if (NOT TARGET Shards::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Shards/ShardsConfig.cmake")
endif()
if (NOT TARGET KokkosCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../KokkosCore/KokkosCoreConfig.cmake")
endif()
if (NOT TARGET KokkosContainers::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../KokkosContainers/KokkosContainersConfig.cmake")
endif()
if (NOT TARGET KokkosAlgorithms::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../KokkosAlgorithms/KokkosAlgorithmsConfig.cmake")
endif()
if (NOT TARGET Sacado::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Sacado/SacadoConfig.cmake")
endif()
if (NOT TARGET KokkosKernels::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../KokkosKernels/KokkosKernelsConfig.cmake")
endif()
if (NOT TARGET TrilinosTribitsLastLib::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/TrilinosTribitsLastLib/TrilinosTribitsLastLibConfig.cmake")
endif()

# Import Intrepid2 targets
include("${CMAKE_CURRENT_LIST_DIR}/Intrepid2Targets.cmake")

## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Intrepid2_EXPORTED_PACKAGE_LIBS_NAMES "intrepid2")

foreach(libname IN LISTS Intrepid2_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Intrepid2::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Intrepid2::${libname}', or better yet,"
      " 'Intrepid2::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Intrepid2'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Intrepid2_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
