# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Thyra clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Thyra requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.17.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Thyra build
## ---------------------------------------------------------------------------

set(Thyra_CXX_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpicxx")

set(Thyra_C_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpicc")

set(Thyra_Fortran_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpif90")
# Deprecated!
set(Thyra_FORTRAN_COMPILER "/usr/lib64/mpi/gcc/openmpi4/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Thyra build
## ---------------------------------------------------------------------------

## Give the build type
set(Thyra_CMAKE_BUILD_TYPE "RelWithDebInfo")

## Set compiler flags, including those determined by build type
set(Thyra_CXX_FLAGS [[ ]])

set(Thyra_C_FLAGS [[  -O2 -Wall -U_FORTIFY_SOURCE -D_FORTIFY_SOURCE=3 -fstack-protector-strong -funwind-tables -fasynchronous-unwind-tables -fstack-clash-protection -Werror=return-type -flto=auto -g -fopenmp ]])

set(Thyra_Fortran_FLAGS [[ ]])
# Deprecated
set(Thyra_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(Thyra_EXTRA_LD_FLAGS [[-lgfortran]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Thyra_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib64/mpi/gcc/openmpi4/lib64")
set(Thyra_BUILD_SHARED_LIBS "ON")

set(Thyra_LINKER /usr/bin/ld)
set(Thyra_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Thyra_INSTALL_DIR "/usr/lib64/mpi/gcc/openmpi4")

## List of package libraries
set(Thyra_LIBRARIES Thyra::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Thyra_MPI_LIBRARIES "")
set(Thyra_MPI_LIBRARY_DIRS "")
set(Thyra_MPI_INCLUDE_DIRS "")
set(Thyra_MPI_EXEC "/usr/lib64/mpi/gcc/openmpi4/bin/mpiexec")
set(Thyra_MPI_EXEC_MAX_NUMPROCS "4")
set(Thyra_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Thyra_ENABLE_ThyraCore ON)
set(Thyra_ENABLE_ThyraEpetraAdapters ON)
set(Thyra_ENABLE_ThyraEpetraExtAdapters ON)
set(Thyra_ENABLE_ThyraTpetraAdapters ON)
set(Thyra_ENABLE_TrilinosTribitsLastLib OFF)

# Exported cache variables
set(Thyra_ENABLE_DEBUG "OFF")
set(HAVE_THYRA_DEBUG "OFF")
set(Thyra_ENABLE_EXPLICIT_INSTANTIATION "ON")
set(HAVE_THYRA_EXPLICIT_INSTANTIATION "ON")
set(Thyra_ENABLE_ME_POLYNOMIAL "ON")
set(HAVE_THYRA_ME_POLYNOMIAL "ON")
set(Thyra_ENABLE_HESSIAN_SUPPORT "ON")
set(Thyra_BUILD_HESSIAN_SUPPORT "ON")

# Include configuration of dependent packages
if (NOT TARGET ThyraCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraCore/ThyraCoreConfig.cmake")
endif()
if (NOT TARGET ThyraEpetraAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraEpetraAdapters/ThyraEpetraAdaptersConfig.cmake")
endif()
if (NOT TARGET ThyraEpetraExtAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraEpetraExtAdapters/ThyraEpetraExtAdaptersConfig.cmake")
endif()
if (NOT TARGET ThyraTpetraAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraTpetraAdapters/ThyraTpetraAdaptersConfig.cmake")
endif()
if (NOT TARGET TrilinosTribitsLastLib::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/TrilinosTribitsLastLib/TrilinosTribitsLastLibConfig.cmake")
endif()

# Import Thyra targets
include("${CMAKE_CURRENT_LIST_DIR}/ThyraTargets.cmake")

## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Thyra_EXPORTED_PACKAGE_LIBS_NAMES "thyracore;thyraepetra;thyraepetraext;thyratpetra")

foreach(libname IN LISTS Thyra_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Thyra::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Thyra::${libname}', or better yet,"
      " 'Thyra::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Thyra'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Thyra_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
